==> node_bsdiff-1.0.0.tgz: package/node_bsdiff.cc <==
#include <v8.h>
#include <node.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "bsdiff.h"

using namespace std;
using namespace node;
using namespace v8;

static Handle<Value> BSDiffAsync (const Arguments&);
static int BSDiff (eio_req *);
static int BSDiff_After (eio_req *);
extern "C" void init (Handle<Object>);

struct bsdiff_request {
	char *old_file, *new_file, *patch_file, err_str[256];
	bool success;
	Persistent<Function> cb;
};

static Handle<Value> BSDiffAsync (const Arguments& args) {
  HandleScope scope;
  const char *usage = "usage: BSDiff(oldfile, newfile, patchfile, callback(err))";
  if (args.Length() != 4) {
    return ThrowException(Exception::Error(String::New(usage)));
  }
  String::Utf8Value *old_file = new String::Utf8Value(args[0]);
  String::Utf8Value *new_file = new String::Utf8Value(args[1]);
  String::Utf8Value *patch_file = new String::Utf8Value(args[2]);
  Local<Function> cb = Local<Function>::Cast(args[3]);

  bsdiff_request *sr = (bsdiff_request *) malloc(sizeof(struct bsdiff_request));
  
  sr->cb = Persistent<Function>::New(cb);
  sr->old_file = (char*) malloc(old_file->length() + 1);
  strncpy(sr->old_file, **old_file, old_file->length() + 1);
  sr->new_file = (char*) malloc(new_file->length() + 1);
  strncpy(sr->new_file, **new_file, new_file->length() + 1);
  sr->patch_file = (char*) malloc(patch_file->length() + 1);
  strncpy(sr->patch_file, **patch_file, patch_file->length() + 1);
  sr->success = false;
  sr->err_str[0] = '\0';
    
  eio_custom(BSDiff, EIO_PRI_DEFAULT, BSDiff_After, sr);
  ev_ref(EV_DEFAULT_UC);
  return Undefined();
}

// this function happens on the thread pool
// doing v8 things in here will make bad happen.
static int BSDiff (eio_req *req) {
  bsdiff_request * sr = (bsdiff_request *)req->data;
  sr->success = bsdiff(sr->old_file, sr->new_file, sr->patch_file);
  return 0;
}

static int BSDiff_After (eio_req *req) {
  HandleScope scope;
  ev_unref(EV_DEFAULT_UC);
  bsdiff_request * sr = (bsdiff_request *)req->data;
  Local<Value> argv[1];
  argv[0] = Local<Value>::New(Boolean::New(sr->success));
  TryCatch try_catch;
  sr->cb->Call(Context::GetCurrent()->Global(), 1, argv);
  if (try_catch.HasCaught()) {
    FatalException(try_catch);
  }
  sr->cb.Dispose();
  free(sr->old_file);
  free(sr->new_file);
  free(sr->patch_file);
  free(sr);
  return 0;
}

extern "C" void init (Handle<Object> target) {
  HandleScope scope;
  NODE_SET_METHOD(target, "BSDiff", BSDiffAsync);
}

==> node_bsdiff-1.0.1.tgz: package/node_bsdiff.cc <==
#include <v8.h>
#include <node.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "bsdiff.h"

using namespace std;
using namespace node;
using namespace v8;

static Handle<Value> BSDiffAsync (const Arguments&);
static int BSDiff (eio_req *);
static int BSDiff_After (eio_req *);
extern "C" void init (Handle<Object>);

struct bsdiff_request {
	char *old_file, *new_file, *patch_file, err_str[256];
	bool success;
	Persistent<Function> cb;
};

static Handle<Value> BSDiffAsync (const Arguments& args) {
  HandleScope scope;
  const char *usage = "usage: BSDiff(oldfile, newfile, patchfile, callback(err))";
  if (args.Length() != 4) {
    return ThrowException(Exception::Error(String::New(usage)));
  }
  String::Utf8Value *old_file = new String::Utf8Value(args[0]);
  String::Utf8Value *new_file = new String::Utf8Value(args[1]);
  String::Utf8Value *patch_file = new String::Utf8Value(args[2]);
  Local<Function> cb = Local<Function>::Cast(args[3]);

  bsdiff_request *sr = (bsdiff_request *) malloc(sizeof(struct bsdiff_request));
  
  sr->cb = Persistent<Function>::New(cb);
  sr->old_file = (char*) malloc(old_file->length() + 1);
  strncpy(sr->old_file, **old_file, old_file->length() + 1);
  sr->new_file = (char*) malloc(new_file->length() + 1);
  strncpy(sr->new_file, **new_file, new_file->length() + 1);
  sr->patch_file = (char*) malloc(patch_file->length() + 1);
  strncpy(sr->patch_file, **patch_file, patch_file->length() + 1);
  sr->success = false;
  sr->err_str[0] = '\0';
    
  eio_custom(BSDiff, EIO_PRI_MIN, BSDiff_After, sr);
  ev_ref(EV_DEFAULT_UC);
  return Undefined();
}

// this function happens on the thread pool
// doing v8 things in here will make bad happen.
static int BSDiff (eio_req *req) {
  bsdiff_request * sr = (bsdiff_request *)req->data;
  sr->success = bsdiff(sr->old_file, sr->new_file, sr->patch_file);
  return 0;
}

static int BSDiff_After (eio_req *req) {
  HandleScope scope;
  ev_unref(EV_DEFAULT_UC);
  bsdiff_request * sr = (bsdiff_request *)req->data;
  Local<Value> argv[1];
  argv[0] = Local<Value>::New(Boolean::New(sr->success));
  TryCatch try_catch;
  sr->cb->Call(Context::GetCurrent()->Global(), 1, argv);
  if (try_catch.HasCaught()) {
    FatalException(try_catch);
  }
  sr->cb.Dispose();
  free(sr->old_file);
  free(sr->new_file);
  free(sr->patch_file);
  free(sr);
  return 0;
}

extern "C" void init (Handle<Object> target) {
  HandleScope scope;
  NODE_SET_METHOD(target, "BSDiff", BSDiffAsync);
}
